<?php
/**
 * @file
 * The help page, providing help information.
 */

/**
 * The help page callback.
 */
function restore_help_page() {
  $output = array();

  $output['contents'] = array(
    '#type' => 'fieldset',
    '#title' => t('Contents'),
    '#collapsible' => TRUE,
    'contents' => array(
      '#type' => 'markup',
      '#markup' => theme('item_list', array(
        'items' => array(
          l(t('What is restore?'), 'admin/config/development/restore/help', array('fragment' => 'what-is-restore')),
          l(t('Why use restore?'), 'admin/config/development/restore/help', array('fragment' => 'why-use-restore')),
          l(t('Operations and elements'), 'admin/config/development/restore/help', array('fragment' => 'operations-elements')),
          l(t('Using restore script files'), 'admin/config/development/restore/help', array('fragment' => 'using-script-files')),
          l(t('Using restore hooks'), 'admin/config/development/restore/help', array('fragment' => 'using-hooks')),
        ),
      )),
    ),
  );

  $output['what-is-restore'] = array(
    '#type' => 'markup',
    '#prefix' => '<h2><a id="what-is-restore"></a>' . t('What is restore?') . '</h2>',
    '#markup' => '<p>' .
    t('Restore is a module aimed at making deployment of sites and modules easier and painless.') . ' ' .
    t('Restore uses restore scripts to provide operation details, which can be stored as a script text file (containing a PHP array), using a hook within your own module (using arrays) or stored in the database.') .
    '</p>',
  );

  $output['why-use-restore'] = array(
    '#type' => 'markup',
    '#prefix' => '<h2><a id="why-use-restore"></a>' . t('Why use restore?') . '</h2>',
    '#markup' => '<p>' .
    t('There are a few other modules that provide similar if not the same functionality as restore, such as !features. The current features provided by restore are as follows:', array(
      '!features' => l(t('Features'), 'http://www.drupal.org/project/features'),
    )) .
    theme('item_list', array(
      'items' => array(
        t('Text/script based restore files, to allow adding and swapping of restore scripts with modules.'),
        t('Hooks to allow for programmatically adding and manipulating restore scripts.'),
        t('Restore scripts can be stored in the database.'),
        t('An admin UI to display the restore scripts and operations with the status of each element of the script(s).'),
        t('An admin UI showing any conflicts with other scripts.'),
        t('An admin UI to compare the the scripts values to the live values.'),
        t('An admin UI to generate, and edit restore scripts.'),
        t('Uses !ctools to provide the operation classes.', array(
          '!ctools' => l(t('ctools'), 'http://www.drupal.org/project/ctools'),
        )),
        t('The operation classes are easily built, and an example skeleton operation is provided.'),
        t('The scripts WILL NOT break if an operation module is not available, rather it just highlights the modules not available and skips it.'),
        t('Whole scripts can be restored/uninstalled, or individual operations and elements can be restored/uninstalled.'),
        t('The idea is not to use IDs (UID, NID, etc) where possible.'),
        t('Optionally automatically restore and/or uninstall scripts when a module is installed or uninstalled.'),
        t('Because the restore scripts are file based and only called when needed, theres a smaller memory and performance hit.'),
      ),
    )) .
    '</p><p>' .
    t('Restore can be used as a deployment tool, for example if your building a Drupal site with several variables, fields, roles and permissions, these can be stored in a restore script(s) and when deployed to another server simply restore the script(s), and if there are any changes, just upload the new restore script and restore it again.') .
    '</p><p>' .
    t('Or it can be simply used as a part of your new module, if your module provides several new fields, roles and rules they can be stored in the module using a hook (or restore script file) and when the module is installed all the fields, roles and rules get installed along with it.') .
    '</p>',
  );

  $output['operations-elements'] = array(
    '#type' => 'markup',
    '#prefix' => '<h2><a id="operations-elements"></a>' . t('Operations and elements') . '</h2>',
    '#markup' => '<p>' .
    t('Restore uses classes called "operations", these are used to manage each of the individual elements and perform the actions required. This way the restore module just needs to look through the script for the operations and details and the execute the required operation class.') . ' ' .
    t('In the script file, under each operation is an array of operation elements. These are the variables, or fields to be restored.') .
    '</p><p>' .
    t('Using this method allows for adding additional operations easy, and also if the operation class does not exist, restore can catch it and skip it with out getting a WSOD.') . ' ' .
    t('Each operation and element can also be restored or uninstalled seperatly without affecting any of the other operations and elements within the script.') .
    '</p>',
  );

  $output['using-script-files'] = array(
    '#type' => 'markup',
    '#prefix' => '<h2><a id="using-script-files"></a>' . t('Using restore script files') . '</h2>',
    '#markup' => '<p>' .
    t('In order to use a generated script file, you still need a module to provide it, this will usually be in either an existing module you have built for the site, or an empty module specific for the scripts. In this module you just need a single hook to tell restore where to look:') .
    '<br /><pre style="padding: 4px 8px; background:#333333; color: #ffffff;">' .
    '/**' . "\n" .
    ' * Implements hook_restore_scripts_location().' . "\n" .
    ' */' . "\n" .
    'function MY_MODULE_restore_scripts_location() {' . "\n" .
    '  // This can be any location required, but in this example we are using [MY MODULE]/restore-scripts.' . "\n" .
    '  return drupal_get_path(\'module\', \'MY_MODULE\') . \'/restore-scripts\';' . "\n" .
    '}' . "\n" .
    '</pre>' .
    t('Once the module has been setup, all you need to do is place any or all of your restore scripts into the defined scripts folder (in the example its [MY MODULE]/restore-scripts) and restore will find them.') . ' ' .
    t('An example module is provided showing this usage, "restore_example_script" in the module/examples folder.') .
    '</p>',
  );

  $output['using-hooks'] = array(
    '#type' => 'markup',
    '#prefix' => '<h2><a id="using-hooks"></a>' . t('Using restore hooks') . '</h2>',
    '#markup' => '<p>' .
    t('Additional to the script files, theres a hook too allow a module to provide its own programatically generated restore script, using "hook_restore_scripts" and returning an array of restore scripts:') . ' ' .
    t('Note that the properties of the operations required depends on the operation classes.') .
    '<br /><pre style="padding: 4px 8px; background:#333333; color: #ffffff;">' .
    '/**' . "\n" .
    '* Implements hook_restore_scripts().' . "\n" .
    '*/' . "\n" .
    'function MY_MODULE_restore_scripts() {' . "\n" .
    '  $scripts = array();' . "\n\n" .
    '  $scripts[\'example_script_1\'] = array(' . "\n" .
    '    // The title for the restore script.' . "\n" .
    '    \'title\' => \'MY MODULES SCRIPT #1\',' . "\n\n" .
    '    // A description for the restore script.' . "\n" .
    '    \'description\' => \'The first restore script provided by MY MODULE.\',' . "\n\n" .
    '    // An array of the operations, indexed by the operations classname.' . "\n" .
    '    \'operations\' => array(' . "\n\n" .
    '      // Some variables.' . "\n" .
    '      \'RestoreVariableOperation\' => array(' . "\n\n" .
    '        array(' . "\n" .
    '          \'name\' => \'some_random_variable_name\',' . "\n" .
    '          \'value\' => \'a random value.\',' . "\n" .
    '        ),' . "\n\n" .
    '        array(' . "\n" .
    '          \'name\' => \'site_name\',' . "\n" .
    '          \'value\' => \'The name of my site.\',' . "\n" .
    '        ),' . "\n\n" .
    '        array(' . "\n" .
    '          \'name\' => \'clean_urls\',' . "\n" .
    '          \'value\' => TRUE,' . "\n" .
    '        ),' . "\n\n" .
    '      ),' . "\n\n" .
    '    ),' . "\n" .
    '  );' . "\n\n" .
    '  return $scripts;' . "\n" .
    '}' . "\n" .
    '</pre>' .
    t('An example module is provided showing this usage, "restore_example_hook" in the module/examples folder.') .
    '</p>',
  );

  return $output;
}
