<?php
/**
 * @file
 * Provides the restore scripts list form.
 */

/**
 * The restore scripts list form callback.
 */
function restore_list_form($form, &$form_state, $script_id = NULL) {
  $form = array();

  // Get scripts.
  $scripts = restore_get_scripts($script_id);
  if ($script_id && $scripts) {
    $form['#script'] = $scripts;
    $scripts = array($script_id => $scripts);
  }

  // Cycle through each script.
  if (count($scripts)) {
    $form['scripts'] = array('#tree' => TRUE);
    foreach ($scripts as $idx => $script) {
      // Get the status of this script.
      $statuses = restore_status($idx);

      // The scripts fieldset.
      $form['scripts'][$idx] = array(
        '#tree' => TRUE,
        '#title' => t('@title <small>(!id)</small>', array(
          '@title' => $script['title'],
          '!id' => l($idx, "admin/config/development/restore/list/{$idx}"),
        )),
        '#script' => $script,
        '#status' => $statuses[$idx]['overall'],
        '#stats' => t('<span class="restore-status-stat restore-status-installed" title="@count item(s) marked as @state">@count</span>', array(
          '@state' => restore_status_display(RESTORE_STATUS_INSTALLED),
          '@count' => $statuses[$idx]['stats']['installed'],
        )) .
        t('<span class="restore-status-stat restore-status-uninstalled" title="@count item(s) marked as @state">@count</span>', array(
          '@state' => restore_status_display(RESTORE_STATUS_UNINSTALLED),
          '@count' => $statuses[$idx]['stats']['uninstalled'],
        )) .
        t('<span class="restore-status-stat restore-status-overriden" title="@count item(s) marked as @state">@count</span>', array(
          '@state' => restore_status_display(RESTORE_STATUS_OVERRIDEN),
          '@count' => $statuses[$idx]['stats']['overriden'],
        )) .
        t('<span class="restore-status-stat restore-status-unknown" title="@count item(s) marked as @state">@count</span>', array(
          '@state' => restore_status_display(RESTORE_STATUS_UNKNOWN),
          '@count' => $statuses[$idx]['stats']['unknown'],
        )),
        '#description' => check_plain($script['description']),
      );

      if ($statuses[$idx]['stats']['conflicts'] > 0) {
        $form['scripts'][$idx]['#conflicts'] = TRUE;
        $form['scripts'][$idx]['#stats'] .= t('<span class="restore-status-stat restore-status-conflicts" title="@count conflict(s)">@count</span>', array(
          '@count' => $statuses[$idx]['stats']['conflicts'],
        ));
      }

      // The scripts action all checkbox.
      $form['scripts'][$idx]['action_all'] = array(
        '#type' => 'checkbox',
        '#title' => t('Action all'),
      );

      // Cycle through each of the operations.
      $form['scripts'][$idx]['operations'] = array('#tree' => TRUE);
      foreach ($script['operations'] as $op => $operation) {
        $form['scripts'][$idx]['operations'][$op] = array('#tree' => TRUE);

        if ($statuses[$idx]['operations'][$op]['stats']['conflicts'] > 0) {
          $form['scripts'][$idx]['operations'][$op]['#conflicts'] = TRUE;
        }

        // The operations type.
        $label = restore_instance_label($op);
        $form['scripts'][$idx]['operations'][$op]['title'] = array(
          '#type' => 'markup',
          '#markup' => $label == FALSE ? t('Unknown operation') : check_plain($label),
        );

        // Cycle through each of the operation actions.
        foreach ($operation as $odx => $action) {
          $form['scripts'][$idx]['operations'][$op][$odx] = array('#tree' => TRUE);

          // The operation action checkbox.
          $form['scripts'][$idx]['operations'][$op][$odx]['action'] = array(
            '#type' => 'checkbox',
          );

          $status = isset($statuses[$idx]['operations'][$op]['actions'][$odx]['status']) ? $statuses[$idx]['operations'][$op]['actions'][$odx]['status'] : 'unknown';
          $status_label = isset($statuses[$idx]['operations'][$op]['actions'][$odx]['status']) ? restore_status_display($statuses[$idx]['operations'][$op]['actions'][$odx]['status']) : t('Unknown');

          // The operations action status.
          $form['scripts'][$idx]['operations'][$op][$odx]['status'] = array(
            '#type' => 'markup',
            '#prefix' => "<span class=\"restore-status restore-status-{$status}\" title=\"" . $status_label . "\">",
            '#suffix' => '</span>',
            '#markup' => $status_label,
          );

          // The operations action details.
          $form['scripts'][$idx]['operations'][$op][$odx]['details'] = array(
            '#type' => 'markup',
            '#prefix' => '<span class="restore-details">',
            '#prefix' => '</span>',
            '#markup' => isset($statuses[$idx]['operations'][$op]['actions'][$odx]['details']) ? $statuses[$idx]['operations'][$op]['actions'][$odx]['details'] : t("Unknown operation '<em>@operation</em>'", array('@operation' => $op)),
          );

          if ($statuses[$idx]['operations'][$op]['stats']['conflicts'] > 0) {
            $form['scripts'][$idx]['operations'][$op]['#conflicts'] = TRUE;

            if (!empty($statuses[$idx]['operations'][$op]['actions'][$odx]['conflicts'])) {
              $form['scripts'][$idx]['operations'][$op][$odx]['#conflicts'] = TRUE;

              $items = array();
              foreach ($statuses[$idx]['operations'][$op]['actions'][$odx]['conflicts'] as $conflict) {
                $items[] = isset($scripts[$conflict['script']]) ? t('@title <em>(@id)</em>', array(
                  '@title' => $scripts[$conflict['script']]['title'],
                  '@id' => $conflict['script'],
                )) : $conflict['script'];
              }

              $form['scripts'][$idx]['operations'][$op][$odx]['details']['#markup'] .= '<div class="restore-conflict-details">' . theme('item_list', array(
                'items' => $items,
                'title' => t('Conflicting with:'),
                'type' => 'ul',
              )) . '</div>';
            }
          }
        }
      }

      // A placeholder for the table output.
      $form['scripts'][$idx]['operations']['table'] = array(
        '#type' => 'markup',
        '#markup' => '',
      );
    }
  }
  else {
    $form['no_scripts'] = array(
      '#type' => 'markup',
      '#prefix' => '<p>',
      '#suffix' => '</p>',
      '#markup' => t('There are currently no available restore scripts.'),
    );
  }

  // Check if the current user has access to the process actions.
  if (count($scripts) && user_access('process restore')) {
    $form['actions'] = array('#type' => 'actions');

    // Restore the selected scripts/actions.
    $form['actions']['restore'] = array(
      '#type' => 'submit',
      '#submit' => array('restore_list_form_restore'),
      '#value' => t('Restore'),
    );

    // Remove the selected scripts/actions.
    $form['actions']['remove'] = array(
      '#type' => 'submit',
      '#submit' => array('restore_list_form_remove'),
      '#value' => t('Remove'),
    );

    if (arg(5)) {
      $form['actions']['delete'] = array(
        '#type' => 'submit',
        '#submit' => array('restore_list_form_delete'),
        '#value' => t('Delete the script'),
        '#attributes' => array(
          'onclick' => 'return confirm("' . t('Are you sure you want to delete this script?') . '");',
        ),
      );

      $form['actions']['export'] = array(
        '#type' => 'submit',
        '#submit' => array('restore_list_form_download'),
        '#value' => t('Download script file'),
      );
    }
  }

  return $form;
}

/**
 * The restore scripts list form 'restore' submit callback.
 */
function restore_list_form_restore($form, &$form_state) {
  $scripts = array();
  foreach ($form_state['values']['scripts'] as $script_id => $script_values) {
    if ($script_values['action_all']) {
      $scripts[$script_id] = array();
    }

    foreach ($script_values['operations'] as $operation => $actions) {
      foreach ($actions as $action_id => $action) {
        if ($action['action']) {
          $scripts[$script_id][$operation][$action_id] = $action_id;
        }
      }
    }
  }

  $results = restore_run_scripts($scripts);
  if ($results['overall']) {
    drupal_flush_all_caches();
    drupal_set_message(t('The scripts/actions have been restored.'));
  }
  else {
    drupal_set_message(t('There was a problem while attempting to restore a script/action.'), 'error');
    foreach ($results['scripts'] as $script_id => $script) {
      if ($script['overall'] == TRUE) {
        continue;
      }

      foreach ($script['operations'] as $operation_id => $operation) {
        if ($operation['overall'] == TRUE) {
          continue;
        }

        foreach ($operation['actions'] as $action_id => $status) {
          if ($status == FALSE) {
            drupal_set_message(t('Failed to restore action #@action in the operation @operation of script @script.', array(
              '@action' => $action_id,
              '@operation' => $operation_id,
              '@script' => $script['title'] . " ({$script_id})",
            )), 'error');
          }
        }
      }
    }
  }
}

/**
 * The restore scripts list form 'remove' submit callback.
 */
function restore_list_form_remove($form, &$form_state) {
  $scripts = array();
  foreach ($form_state['values']['scripts'] as $script_id => $script_values) {
    if ($script_values['action_all']) {
      $scripts[$script_id] = array();
    }

    foreach ($script_values['operations'] as $operation => $actions) {
      foreach ($actions as $action_id => $action) {
        if ($action['action']) {
          $scripts[$script_id][$operation][$action_id] = $action_id;
        }
      }
    }
  }

  $results = restore_run_scripts($scripts, 'remove');
  if ($results['overall']) {
    drupal_flush_all_caches();
    drupal_set_message(t('The scripts/actions have been removed.'));
  }
  else {
    drupal_set_message(t('There was a problem while attempting to remove a script/action.'), 'error');
    foreach ($results['scripts'] as $script_id => $script) {
      if ($script['overall'] == TRUE) {
        continue;
      }

      foreach ($script['operations'] as $operation_id => $operation) {
        if ($operation['overall'] == TRUE) {
          continue;
        }

        foreach ($operation['actions'] as $action_id => $status) {
          if ($status == FALSE) {
            drupal_set_message(t('Failed to remove action #@action in the operation @operation of script @script.', array(
              '@action' => $action_id,
              '@operation' => $operation_id,
              '@script' => $script['title'] . " ({$script_id})",
            )), 'error');
          }
        }
      }
    }
  }
}

/**
 * The restore scripts list form 'delete' submit callback.
 */
function restore_list_form_delete($form, &$form_state) {
  restore_delete_database_script($form['#script']);

  drupal_set_message(t('The restore script <em>@title</em> has been deleted.', array(
    '@title' => $form['#script']['title'],
  )));

  $form_state['redirect'] = 'admin/config/development/restore/list';
}

/**
 * The restore scripts list form 'download' submit callback.
 */
function restore_list_form_download($form, &$form_state) {
  $filename = drupal_strtolower(str_replace(' ', '_', $form['#script']['title'])) . '.script';

  header('Cache-Control: public');
  header('Content-Type: application/octet-stream');
  header('Cache-Control: no-store, no-cache, must-revalidate');
  header('Cache-Control: post-check=0, pre-check=0');
  header("Content-Disposition: attachment; filename=\"$filename\";");
  header('Content-Transfer-Encoding: binary');

  print restore_generate_script_to_text($form['#script']);

  drupal_exit();
}
