<?php
/**
 * @file
 * Provides the restore scripts list form theme.
 */

/**
 * The restore scripts list form theme callback.
 */
function theme_restore_list_form($variables) {
  drupal_add_css(drupal_get_path('module', 'restore') . '/assets/css/restore.ui.css');
  drupal_add_js(drupal_get_path('module', 'restore') . '/assets/js/restore.ui.js');

  $form = $variables['form'];

  $header = array(
    array(
      'data' => t('Action'),
      'width' => 24,
    ),
    array(
      'data' => t('Status'),
      'width' => 100,
    ),
    t('Operation action details'),
  );

  $rows = array();

  if (isset($form['scripts'])) {
    foreach (element_children($form['scripts']) as $idx) {
      $status_label = restore_status_display($form['scripts'][$idx]['#status']);

      $classes = array('restore-script-title', "restore-status-{$form['scripts'][$idx]['#status']}");
      $label = "<span class=\"restore-script-status\" title=\"{$status_label}\">{$status_label}</span> <strong>{$form['scripts'][$idx]['#title']}</strong> <div class=\"restore-script-stats\">{$form['scripts'][$idx]['#stats']}</div>";

      if (isset($form['scripts'][$idx]['#conflicts'])) {
        $classes[] = 'restore-script-conflicts';
        $label = '<span class="restore-script-status-conflicts" title="' . t('Conflicts') . '">' . t('Conflicts') . '</span> ' . $label;
      }

      $rows[] = array(
        'data' => array(
          array(
            'data' => $label,
            'colspan' => 3,
            'header' => TRUE,
          ),
        ),
        'no_striping' => TRUE,
        'class' => $classes,
      );

      $rows[] = array(
        'data' => array(
          array(
            'data' => drupal_render($form['scripts'][$idx]['action_all']),
            'colspan' => 3,
            'header' => TRUE,
          ),
        ),
        'no_striping' => TRUE,
      );

      $rows[] = array(
        'data' => array(
          array(
            'data' => "<div class=\"restore-script-description\">{$form['scripts'][$idx]['#description']}</div>",
            'colspan' => 3,
          ),
        ),
        'no_striping' => TRUE,
        'class' => array('restore-script-details'),
      );

      foreach (element_children($form['scripts'][$idx]['operations']) as $op) {
        if ($op == 'table') {
          continue;
        }

        $classes = array('restore-operation');
        if (isset($form['scripts'][$idx]['operations'][$op]['#conflicts'])) {
          $classes[] = 'restore-operation-conflicts';
        }

        $rows[] = array(
          'data' => array(
            array(
              'data' => drupal_render($form['scripts'][$idx]['operations'][$op]['title']),
              'colspan' => 3,
              'header' => TRUE,
            ),
          ),
          'no_striping' => TRUE,
          'class' => $classes,
        );

        foreach (element_children($form['scripts'][$idx]['operations'][$op]) as $odx) {
          if ($odx === 'title') {
            continue;
          }

          $classes = array('restore-action');
          if (isset($form['scripts'][$idx]['operations'][$op][$odx]['#conflicts'])) {
            $classes[] = 'restore-action-conflicts';
          }

          $rows[] = array(
            'data' => array(
              drupal_render($form['scripts'][$idx]['operations'][$op][$odx]['action']),
              drupal_render($form['scripts'][$idx]['operations'][$op][$odx]['status']),
              drupal_render($form['scripts'][$idx]['operations'][$op][$odx]['details']),
            ),
            'class' => $classes,
          );
        }
      }
    }
  }

  $output = theme('table', array(
    'rows' => $rows,
    'header' => $header,
    'attributes' => array(
      'class' => array(arg(5) ? 'restore-script' : 'restore-scripts'),
    ),
  ));

  $output .= drupal_render_children($form);
  return $output;
}
