<?php
/**
 * @file
 * Operation plugin for path alias restore operations.
 */

$plugin = array(
  'label' => t('Path alias'),
  'handler' => array(
    'class' => 'RestoreOperationsAliasOperation',
  ),
);

class RestoreOperationsAliasOperation extends RestoreOperation {
  /** @var array Static storage for the variables for generate. */
  protected static $aliases;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Path alias '<em>@alias</em>' => '<em>@source</em>'.", array(
      '@alias' => $action['alias'],
      '@source' => $action['source'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    return restore_operations_alias_check($action['alias']) ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    if (restore_operations_alias_check($action['alias'])) {
      restore_operations_alias_delete($action['alias']);
    }

    restore_operations_alias_add($action['source'], $action['alias']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    restore_operations_alias_delete($action['alias']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['alias'] !== $action['alias']) {
        continue;
      }

      $results[$action_id] = t("Path alias '<em>@alias</em>' => '<em>@source</em>'.", array(
        '@alias' => $_action['alias'],
        '@source' => $_action['source'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $value = restore_operations_alias_get($action['alias']);

    return array(
      'current_value' => $value ? "{$value['source']} => {$value['alias']}" : '',
      'restore_value' => "{$action['source']} => {$action['alias']}",
      'match' => $value ? ($value['source'] == $action['source'] && $value['alias'] == $action['alias']) : NULL,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $alias = restore_operations_alias_get($id);

    if ($alias) {
      return t("Path alias '<em>@alias</em>' => '<em>@source</em> [@lang]'.", array(
        '@alias' => $alias['alias'],
        '@source' => $alias['source'],
        '@lang' => $alias['language'],
      ));
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$aliases)) {
      $dbq = db_select('url_alias', 'ua')
        ->fields('ua')
        ->execute();

      while ($row = $dbq->fetchAssoc()) {
        static::$aliases[$row['alias']] = t('@alias => @source [@lang]', array(
          '@alias' => $row['alias'],
          '@source' => $row['source'],
          '@lang' => $row['language'],
        ));
      }
    }

    return static::$aliases;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $alias = restore_operations_alias_get($id);
    if ($alias) {
      return array(
        'name' => $id,
        'alias' => $alias['alias'],
        'source' => $alias['source'],
        'language' => $alias['language'],
      );
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $details = $this->detailsFromId($action['alias']);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $action['alias'],
      'details' => $details,
    );
  }
}
