<?php
/**
 * @file
 * Operation plugin for block restore operations.
 */

$plugin = array(
  'label' => t('Blocks'),
  'handler' => array(
    'class' => 'RestoreOperationsBlockOperation',
  ),
);

class RestoreOperationsBlockOperation extends RestoreOperation {
  /** @var array Static storage for the blocks for generate. */
  protected static $blocks;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return isset($action['custom_block']) ? t("Custom block '<em>@info</em>', on theme '<em>@theme</em>'.", array(
      '@info' => $action['custom_block']['info'],
      '@theme' => $action['theme'],
    )) : t("Block from module '<em>@module</em>', with delta '<em>@delta</em>', on theme '<em>@theme</em>'.", array(
      '@module' => $action['module'],
      '@delta' => $action['delta'],
      '@theme' => $action['theme'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    if (isset($action['custom_block'])) {
      $action['delta'] = db_query('SELECT bid FROM {block_custom} WHERE info = :info', array(':info' => $action['custom_block']['info']))->fetchField();
    }

    $block = db_query('SELECT * FROM {block} WHERE module = :module AND delta = :delta and theme = :theme', array(
      ':module' => $action['module'],
      ':delta' => $action['delta'],
      ':theme' => $action['theme'],
    ))->fetchAssoc();

    if ($block) {
      unset($block['bid']);
      $matches = TRUE;

      foreach ($block as $key => $value) {
        if ($action[$key] != $value) {
          $matches = FALSE;
          break;
        }
      }

      if ($matches && isset($action['custom_block'])) {
        $custom_block = block_custom_block_get($block['delta']);
        if ($custom_block) {
          unset($custom_block['bid']);
          foreach ($custom_block as $key => $value) {
            if ($action['custom_block'][$key] != $value) {
              $matches = FALSE;
              break;
            }
          }
        }
        else {
          $matches = FALSE;
        }
      }
    }

    return $block ? ($matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    if (isset($action['custom_block'])) {
      $bid = db_query('SELECT bid FROM {block_custom} WHERE info = :info', array(':info' => $action['custom_block']['info']))->fetchField();

      $fields = array(
        'body' => $action['custom_block']['body'],
        'info' => $action['custom_block']['info'],
        'format' => $action['custom_block']['format'],
      );

      $bid ?
        db_update('block_custom')->fields($fields)->condition('bid', $bid)->execute() :
        db_insert('block_custom')->fields($fields)->execute();

      $action['delta'] = db_query('SELECT bid FROM {block_custom} WHERE info = :info', array(':info' => $action['custom_block']['info']))->fetchField();
    }

    $bid = db_query('SELECT bid FROM {block} WHERE module = :module AND delta = :delta AND theme = :theme', array(
      ':module' => $action['module'],
      ':delta' => $action['delta'],
      ':theme' => $action['theme'],
    ))->fetchField();

    $fields = array(
      'module' => $action['module'],
      'delta' => $action['delta'],
      'theme' => $action['theme'],
      'status' => $action['status'],
      'weight' => $action['weight'],
      'region' => $action['region'],
      'custom' => $action['custom'],
      'visibility' => $action['visibility'],
      'pages' => $action['pages'],
      'title' => $action['title'],
      'cache' => $action['cache'],
    );

    $bid ?
      db_update('block')->fields($fields)->condition('bid', $bid)->execute() :
      db_insert('block')->fields($fields)->execute();

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    if (isset($action['custom_block'])) {
      $action['delta'] = db_query('SELECT bid FROM {block_custom} WHERE info = :info', array(':info' => $action['custom_block']['info']))->fetchField();

      db_delete('block')
        ->condition('module', 'block')
        ->condition('delta', $action['delta'])
        ->execute();

      db_delete('block_custom')
        ->condition('bid', $action['delta'])
        ->execute();

      db_delete('block_role')
        ->condition('module', 'block')
        ->condition('delta', $action['delta'])
        ->execute();
    }
    else {
      $bid = db_query('SELECT bid FROM {block} WHERE module = :module AND delta = :delta and theme = :theme', array(
        ':module' => $action['module'],
        ':delta' => $action['delta'],
        ':theme' => $action['theme'],
      ))->fetchAssoc();

      db_delete('block')
        ->condition('module', $action['module'])
        ->condition('delta', $action['delta'])
        ->condition('delta', $action['theme'])
        ->execute();
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['module'] !== $action['module'] &&
        (isset($action['custom_block']) && isset($_action['custom_block']) ? $_action['custom_block']['info'] !== $action['custom_block']['info'] : $_action['delta'] !== $action['delta']) &&
        $_action['theme'] !== $action['theme']
      )) {
        continue;
      }

      $results[$action_id] = isset($_action['custom_block']) ? t("Custom block '<em>@info</em>', on theme '<em>@theme</em>'.", array(
        '@info' => $_action['custom_block']['info'],
        '@theme' => $_action['theme'],
      )) : t("Block from module '<em>@module</em>', with delta '<em>@delta</em>', on theme '<em>@theme</em>'.", array(
        '@module' => $_action['module'],
        '@delta' => $_action['delta'],
        '@theme' => $_action['theme'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    if (isset($action['custom_block'])) {
      $action['delta'] = db_query('SELECT bid FROM {block_custom} WHERE info = :info', array(':info' => $action['custom_block']['info']))->fetchField();
    }

    $block = db_query('SELECT * FROM {block} WHERE module = :module AND delta = :delta and theme = :theme', array(
      ':module' => $action['module'],
      ':delta' => $action['delta'],
      ':theme' => $action['theme'],
    ))->fetchAssoc();

    if ($block) {
      unset($block['bid']);
      $matches = TRUE;

      foreach ($block as $key => $value) {
        if ($action[$key] != $value) {
          $matches = FALSE;
          break;
        }
      }

      if ($matches && isset($action['custom_block'])) {
        $custom_block = block_custom_block_get($block['delta']);
        if ($custom_block) {
          unset($custom_block['bid']);
          foreach ($custom_block as $key => $value) {
            if ($action['custom_block'][$key] != $value) {
              $matches = FALSE;
              break;
            }
          }

          $block['custom_block'] = $custom_block;
        }
        else {
          $matches = FALSE;
        }
      }
    }

    return array(
      'current_value' => $block ? $block : NULL,
      'restore_value' => $action,
      'match' => $block && $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $block = db_query('SELECT * FROM {block} WHERE bid = :bid', array(':bid' => $id))->fetchAssoc();
    if (!$block) {
      return FALSE;
    }

    if ($block['module'] == 'block') {
      $custom_block = block_custom_block_get($block['delta']);
      if ($custom_block) {
        return t("Custom block '<em>@info</em>', on theme '<em>@theme</em>'.", array(
          '@info' => $custom_block['info'],
          '@theme' => $block['theme'],
        ));
      }
    }

    return t("Block from module '<em>@module</em>', with delta '<em>@delta</em>', on theme '<em>@theme</em>'.", array(
      '@module' => $block['module'],
      '@delta' => $block['delta'],
      '@theme' => $block['theme'],
    ));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$blocks)) {
      $custom_blocks = db_select('block_custom', 'bc')
        ->fields('bc')
        ->execute()
        ->fetchAllAssoc('bid', PDO::FETCH_ASSOC);

      $blocks = db_select('block', 'bl')
        ->fields('bl')
        ->execute();

      while ($row = $blocks->fetchAssoc()) {
        $info = ($row['module'] == 'block') ? t("Custom block '<em>@info</em>', on theme '<em>@theme</em>'.", array(
          '@info' => $custom_blocks[$row['delta']]['info'],
          '@theme' => $row['theme'],
        )) : t("Block from module '<em>@module</em>', with delta '<em>@delta</em>', on theme '<em>@theme</em>'.", array(
          '@module' => $row['module'],
          '@delta' => $row['delta'],
          '@theme' => $row['theme'],
        ));

        static::$blocks[$row['bid']] = $info;
      }
    }

    return static::$blocks;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $block = db_query('SELECT * FROM {block} WHERE bid = :bid', array(':bid' => $id))->fetchAssoc();
    if (!$block) {
      return FALSE;
    }

    $bid = (int) $block['delta'];
    $result = $block;
    unset($result['bid']);

    if ($block['module'] == 'block') {
      $custom_block = block_custom_block_get($bid);
      if ($custom_block) {
        $result['custom_block'] = array(
          'info' => $custom_block['info'],
          'body' => $custom_block['body'],
          'format' => $custom_block['format'],
        );
      }
    }

    return $result;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $bid = db_query('SELECT bid FROM {block} WHERE module = :module AND delta = :delta and theme = :theme', array(
      ':module' => $action['module'],
      ':delta' => $action['delta'],
      ':theme' => $action['theme'],
    ))->fetchAssoc();

    if (!$bid) {
      return FALSE;
    }

    $details = $this->detailsFromId($bid['bid']);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $bid['bid'],
      'details' => $details,
    );
  }
}
