<?php
/**
 * @file
 * Operation plugin for custom breadcrumbs restore operations.
 */

$plugin = array(
  'label' => t('Custom breadcrumbs'),
  'handler' => array(
    'class' => 'RestoreOperationsCustombreadcrumbsOperation',
  ),
);

class RestoreOperationsCustombreadcrumbsOperation extends RestoreOperation {
  /** @var array Static storage for the breadcrumbs for generate. */
  protected static $breadcrumbs;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Custom breadcrumb '<em>@name</em>' (@type)", array(
      '@name' => $action['name'],
      '@type' => $action['breadcrumb_type'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $breadcrumb = restore_operations_custombreadcrumbs_load_by_type($action['name'], $action['breadcrumb_type']);
    if ($breadcrumb) {
      $matches = TRUE;
      $keys = array_keys($action);
      foreach ($keys as $key) {
        if (!isset($breadcrumb->$key) || $breadcrumb->$key != $action[$key]) {
          $matches = FALSE;
          break;
        }
      }
    }

    return $breadcrumb ? ($matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    $module = restore_operations_custombreadcrumbs_module($action['breadcrumb_type']);
    if (!$module) {
      return FALSE;
    }

    $original = restore_operations_custombreadcrumbs_load_by_type($action['name'], $action['breadcrumb_type']);
    $breadcrumb = (object) $action;

    if ($original) {
      $breadcrumb->bid = $original->bid;
    }

    _custom_breadcrumbs_save_breadcrumb($module['module'], $module['key'], $breadcrumb);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $module = restore_operations_custombreadcrumbs_module($action['breadcrumb_type']);
    if (!$module) {
      return FALSE;
    }

    $breadcrumb = restore_operations_custombreadcrumbs_load_by_type($action['name'], $action['breadcrumb_type']);
    if (!$breadcrumb) {
      return FALSE;
    }

    _custom_breadcrumbs_delete_breadcrumb($module['module'], $module['key'], $breadcrumb->bid);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['name'] !== $action['name'] &&
        $_action['breadcrumb_type'] !== $action['breadcrumb_type']
      )) {
        continue;
      }

      $results[$action_id] = t("Custom breadcrumb '<em>@name</em>' (@type)", array(
        '@name' => $_action['name'],
        '@type' => $_action['breadcrumb_type'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $breadcrumb = restore_operations_custombreadcrumbs_load_by_type($action['name'], $action['breadcrumb_type']);
    if (!$breadcrumb) {
      return FALSE;
    }

    $matches = TRUE;
    $keys = array_keys($action);
    foreach ($keys as $key) {
      if (!isset($breadcrumb->$key) || $breadcrumb->$key != $action[$key]) {
        $matches = FALSE;
        break;
      }
    }

    return array(
      'current_value' => (array) $breadcrumb,
      'restore_value' => $action,
      'match' => $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $breadcrumb = restore_operations_custombreadcrumbs_load($id);
    if ($breadcrumb) {
      return t("Custom breadcrumb '<em>@name</em>' (@type)", array(
        '@name' => $breadcrumb->name,
        '@type' => $breadcrumb->breadcrumb_type,
      ));
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$breadcrumbs)) {
      $breadcrumbs = _custom_breadcrumbs_load_all_breadcrumbs();
      foreach ($breadcrumbs as $breadcrumb) {
        static::$breadcrumbs[$breadcrumb->bid] = t("Custom breadcrumb '<em>@name</em>' (@type)", array(
          '@name' => $breadcrumb->name,
          '@type' => $breadcrumb->breadcrumb_type,
        ));
      }
    }

    return static::$breadcrumbs;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $breadcrumb = restore_operations_custombreadcrumbs_load($id);

    if ($breadcrumb) {
      $bc = (array) $breadcrumb;
      unset($bc['bid']);
      return $bc;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $breadcrumb = restore_operations_custombreadcrumbs_load_by_type($action['name'], $action['breadcrumb_type']);
    if (!$breadcrumb) {
      return FALSE;
    }

    $details = $this->detailsFromId($breadcrumb->bid);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $breadcrumb->bid,
      'details' => $details,
    );
  }
}
