<?php
/**
 * @file
 * Operation plugin for filter format restore operations.
 */

$plugin = array(
  'label' => t('Filter formats'),
  'handler' => array(
    'class' => 'RestoreOperationsFilterFormatOperation',
  ),
);

class RestoreOperationsFilterFormatOperation extends RestoreOperation {
  /** @var array Static storage for the filter formats for generate. */
  protected static $formats;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return filter_xss($action['name']);
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $format = restore_operations_filter_format_load($action['format']);

    $matches = FALSE;
    if ($format) {
      $matches = TRUE;

      if (!(
        $action['name'] == $format->name &&
        $action['cache'] == $format->cache &&
        $action['status'] == $format->status &&
        $action['weight'] == $format->weight
      )) {
        $matches = FALSE;
      }

      if ($matches && count($format->filters) == count($action['filters'])) {
        foreach ($action['filters'] as $filter_name => $filter) {
          if ($format->filters[$filter_name] != $filter) {
            $matches = FALSE;
            break;
          }
        }
      }
    }

    return $format ? ($matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    filter_format_save((object) $action);

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $format = filter_format_load($action['format']);
    if ($format) {
      filter_format_disable((object) $format);
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!($_action['format'] !== $action['format'])) {
        continue;
      }

      $results[$action_id] = $this->details($_action);
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $format = restore_operations_filter_format_load($action['format']);

    $matches = FALSE;
    if ($format) {
      $matches = TRUE;

      if (!(
        $action['name'] == $format->name &&
        $action['cache'] == $format->cache &&
        $action['status'] == $format->status &&
        $action['weight'] == $format->weight
      )) {
        $matches = FALSE;
      }

      if ($matches && count($format->filters) == count($action['filters'])) {
        foreach ($action['filters'] as $filter_name => $filter) {
          if ($format->filters[$filter_name] != $filter) {
            $matches = FALSE;
            break;
          }
        }
      }
    }

    return array(
      'current_value' => $format ? $format : NULL,
      'restore_value' => $action,
      'match' => $format && $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $format = filter_format_load($id);
    if (!$format) {
      return FALSE;
    }

    return filter_xss($format->name);
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$formats)) {
      $formats = filter_formats();

      foreach ($formats as $name => $format) {
        static::$formats[$name] = filter_xss($format->name);
      }
    }

    return static::$formats;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $format = restore_operations_filter_format_load($id);
    if (!$format) {
      return FALSE;
    }

    return (array) $format;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $format = restore_operations_filter_format_load($action['format']);
    if (!$format) {
      return FALSE;
    }

    $details = $this->detailsFromId($format->format);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $format->format,
      'details' => $details,
    );
  }
}
