<?php
/**
 * @file
 * Operation plugin for menu links restore operations.
 */

$plugin = array(
  'label' => t('Menu links'),
  'handler' => array(
    'class' => 'RestoreOperationsMenuLinkOperation',
  ),
);

class RestoreOperationsMenuLinkOperation extends RestoreOperation {
  /** @var array Static storage for the menu links for generate. */
  protected static $links;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Menu link '<em>@title</em>' => '<em>@path</em>'.", array(
      '@title' => $action['title'],
      '@path' => $action['path'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $link = restore_operations_menu_menulink_get($action['path'], $action['title'], $action['menu_name']);

    return $link ? (
      $link['link_path'] == $action['path'] &&
      $link['link_title'] == $action['title'] &&
      $link['weight'] == $action['weight']) ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    restore_operations_menu_menulink_add($action['title'], $action['path'], $action['menu_name'], isset($action['weight']) ? $action['weight'] : 0);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    restore_operations_menu_menulink_delete($action['path'], $action['title'], $action['menu_name']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['title'] !== $action['title'] &&
        $_action['path'] !== $action['path']
      )) {
        continue;
      }

      $results[$action_id] = t("Menu link '<em>@title</em>' => '<em>@path</em>'.", array(
        '@title' => $_action['title'],
        '@path' => $_action['path'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $link = restore_operations_menu_menulink_get($action['path'], $action['title'], $action['menu_name']);
    if (!$link) {
      return FALSE;
    }

    return array(
      'current_value' => array(
        'title' => $link['link_title'],
        'path' => $link['link_path'],
        'weight' => $link['weight'],
      ),
      'restore_value' => array(
        'title' => $action['title'],
        'path' => $action['path'],
        'weight' => $action['weight'],
      ),
      'match' => (
        $link['link_path'] == $action['path'] &&
        $link['link_title'] == $action['title'] &&
        $link['weight'] == $action['weight']
      ),
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $link = db_select('menu_links', 'ml')
      ->fields('ml', array('menu_name', 'link_path', 'link_title'))
      ->condition('ml.mlid', $id)
      ->execute()
      ->fetchAssoc();

    return t("Link '<em>@name</em>' => '<em>@path</em>' from menu '<em>@menu_name</em>'.", array(
      '@name' => $link['link_title'],
      '@path' => $link['link_path'],
      '@menu_name' => $link['menu_name'],
    ));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$links)) {
      $links = db_select('menu_links', 'ml')
        ->fields('ml', array('mlid', 'menu_name', 'link_path', 'link_title'))
        ->execute()
        ->fetchAllAssoc('mlid', PDO::FETCH_ASSOC);

      foreach ($links as $mlid => $link) {
        static::$links[$mlid] = t("Link '<em>@name</em>' => '<em>@path</em>' from menu '<em>@menu_name</em>'.", array(
          '@name' => $link['link_title'],
          '@path' => $link['link_path'],
          '@menu_name' => $link['menu_name'],
        ));
      }
    }

    return static::$links;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $link = menu_link_load($id);

    if ($link) {
      $result = array(
        'title' => $link['link_title'],
        'path' => $link['link_path'],
        'menu_name' => $link['menu_name'],
        'weight' => $link['weight'],
      );

      return $result;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $link = restore_operations_menu_menulink_get($action['path'], $action['title'], $action['menu_name']);
    if ($link) {
      return array(
        'id' => $link['mlid'],
        'details' => t("Link '<em>@name</em>' => '<em>@path</em>' from menu '<em>@menu_name</em>'.", array(
          '@name' => $link['link_title'],
          '@path' => $link['link_path'],
          '@menu_name' => $link['menu_name'],
        )),
      );
    }

    return FALSE;
  }
}
