<?php
/**
 * @file
 * Operation plugin for node content type restore operations.
 */

$plugin = array(
  'label' => t('Content types'),
  'handler' => array(
    'class' => 'RestoreOperationsNodeTypeOperation',
  ),
);

// @TODO Should the fields for the content type be bundled?
class RestoreOperationsNodeTypeOperation extends RestoreOperation {
  /** @var array Static storage for the content types for generate. */
  protected static $contentTypes;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t('<strong>@name</strong>: !description', array(
      '@name' => $action['name'],
      '!description' => filter_xss($action['description']),
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $type = node_type_get_type($action['type']);

    if ($type) {
      $matches = TRUE;

      foreach ($action as $key => $value) {
        if ($type->$key != $value) {
          $matches = FALSE;
          break;
        }
      }
    }

    return $type ? ($matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    $type = node_type_get_type($action['type']);
    if (!$type) {
      $type = (object) array(
        'type' => $action['type'],
      );
    }

    $type->name = $action['name'];
    $type->base = $action['base'];
    $type->description = $action['description'];
    $type->help = $action['help'];
    $type->custom = $action['custom'];
    $type->modified = $action['modified'];
    $type->locked = $action['locked'];
    $type->disabled = $action['disabled'];
    $type->has_title = $action['has_title'];
    $type->title_label = $action['title_label'];

    node_type_save($type);

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $type = node_type_get_type($action['type']);
    if ($type) {
      node_type_delete($type->type);
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!($_action['type'] !== $action['type'])) {
        continue;
      }

      $results[$action_id] = $this->details($_action);
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $type = node_type_get_type($action['type']);

    if ($type) {
      $matches = TRUE;

      foreach ($action as $key => $value) {
        if ($type->$key != $value) {
          $matches = FALSE;
          break;
        }
      }
    }

    return array(
      'current_value' => $type ? $type : NULL,
      'restore_value' => $action,
      'match' => $type && $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $type = node_type_get_type($id);
    if (!$type) {
      return FALSE;
    }

    return t('<strong>@name</strong>: !description', array(
      '@name' => $type->name,
      '!description' => filter_xss($type->description),
    ));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$contentTypes)) {
      $types = node_type_get_types();

      foreach ($types as $id => $type) {
        static::$contentTypes[$id] = t('<strong>@name</strong>: !description', array(
          '@name' => $type->name,
          '!description' => filter_xss($type->description),
        ));
      }
    }

    return static::$contentTypes;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $type = node_type_get_type($id);
    if (!$type) {
      return FALSE;
    }

    $result = array(
      'type' => $type->type,
      'name' => $type->name,
      'base' => $type->base,
      'description' => $type->description,
      'help' => $type->help,
      'custom' => $type->custom,
      'modified' => $type->modified,
      'locked' => $type->locked,
      'disabled' => $type->disabled,
      'has_title' => $type->has_title,
      'title_label' => $type->title_label,
    );

    return $result;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $type = node_type_get_type($action['type']);
    if (!$type) {
      return FALSE;
    }

    $details = $this->detailsFromId($type->type);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $type->type,
      'details' => $details,
    );
  }
}
