<?php
/**
 * @file
 * The diff page.
 */

/**
 * The diff page callback.
 */
function restore_diff_page($script_id = NULL) {
  drupal_add_css(drupal_get_path('module', 'restore') . '/assets/css/restore.ui.css');
  drupal_add_js(drupal_get_path('module', 'restore') . '/assets/js/restore.ui.js');
  drupal_add_css(drupal_get_path('module', 'restore_diff') . '/assets/css/restore-diff.ui.css');

  // Get scripts.
  $scripts = restore_get_scripts($script_id);
  if ($script_id && $scripts) {
    $scripts = array($script_id => $scripts);
  }

  if ($script_id) {
    if (!isset($scripts[$script_id])) {
      drupal_set_message(t('Failed to find a restore script with the name of <em>@script_name</em>.', array('@script_name' => $script_id)), 'error');
      return drupal_goto('admin/config/development/restore/diff');
    }
  }

  $output = '';

  $header = array(
    t('Status'),
    array(
      'data' => t('Details'),
      'width' => '30%',
    ),
    array(
      'data' => t('Current value'),
      'width' => '30%',
    ),
    array(
      'data' => t('Restore value'),
      'width' => '30%',
    ),
  );

  $rows = array();

  // Cycle through each script.
  if (count($scripts)) {
    foreach ($scripts as $script_id => $script) {
      $matches = $mismatches = $unknowns = $undefined = 0;
      $title_row = count($rows);

      $rows[] = array(
        'data' => array(
          array(
            'data' => t("<strong>@title</strong> <small>(@id)</small>", array(
              '@title' => $script['title'],
              '@id' => $script_id,
            )),
            'colspan' => 4,
            'header' => TRUE,
          ),
        ),
        'no_striping' => TRUE,
        'class' => array('restore-script-title'),
      );

      foreach ($script['operations'] as $operation_id => $operation) {
        $instance = restore_instance($operation_id);
        $label = restore_instance_label($operation_id);

        $rows[] = array(
          'data' => array(
            array(
              'data' => $label == FALSE ? t('Unknown operation') : check_plain($label),
              'colspan' => 4,
              'header' => TRUE,
            ),
          ),
          'no_striping' => TRUE,
        );

        foreach ($operation as $action_id => $action) {
          $match = $instance ? $instance->diff($action) : array(
            'current_value' => t('<em>Unable to find a diff class.</em>'),
            'restore_value' => t('<em>Unable to find a diff class.</em>'),
            'match' => NULL,
          );

          if (!$instance) {
            $class = 'restore-diff-unknown';
            $label = t('Unknown');
            $unknowns++;
          }
          elseif ($match['match']) {
            $class = 'restore-diff-match';
            $label = t('Match');
            $matches++;
          }
          elseif ($match['match'] == NULL) {
            $class = 'restore-diff-undefined';
            $label = t('Undefined');
            $undefined++;
          }
          else {
            $class = 'restore-diff-mismatch';
            $label = t('Mismatch');
            $mismatches++;
          }

          $rows[] = array(
            'data' => array(
              '<span class="restore-diff-status">' . $label . '</span>',
              '<span class="restore-diff-details">' . ($instance ? $instance->details($action) : t('Unknown operation')) . '</span>',
              '<span class="restore-diff-current-value">' . restore_diff_value_display($match['current_value']) . '</span>',
              '<span class="restore-diff-restore-value">' . restore_diff_value_display($match['restore_value']) . '</span>',
            ),
            'class' => array('restore-diff', $class),
          );
        }
      }

      $rows[$title_row]['data'][0]['data'] .= '<div class="restore-script-stats">' . t('<span class="restore-status-stat restore-diff-match" title="@count item(s) marked as @state">@count</span>', array(
        '@state' => t('Match'),
        '@count' => $matches,
      )) .
      t('<span class="restore-status-stat restore-diff-mismatch" title="@count item(s) marked as @state">@count</span>', array(
        '@state' => t('Mismatch'),
        '@count' => $mismatches,
      )) .
      t('<span class="restore-status-stat restore-diff-unknown" title="@count item(s) marked as @state">@count</span>', array(
        '@state' => t('Unknown'),
        '@count' => $unknowns,
      )) .
      t('<span class="restore-status-stat restore-diff-undefined" title="@count item(s) marked as @state">@count</span>', array(
        '@state' => t('Undefined'),
        '@count' => $undefined,
      )) . '</div>';
    }
  }

  $output .= theme('table', array(
    'rows' => $rows,
    'header' => $header,
    'attributes' => array(
      'class' => array(arg(5) ? 'restore-script' : 'restore-scripts'),
    ),
  ));

  return $output;
}
