<?php
/**
 * @file
 * The generate form.
 */

/**
 * The generate form callback.
 */
function restore_generate_form($form, &$form_state, $script_id = NULL) {
  $form = array();

  if ($script_id !== NULL && ($script = restore_get_scripts($script_id)) !== FALSE && !isset($form_state['storage']['script'])) {
    $form_state['storage']['script'] = restore_generate_from_script($script);
  }

  // Create the stored script if its not available.
  if (!isset($form_state['storage']['script'])) {
    $form_state['storage']['script'] = array(
      'title' => '',
      'description' => '',
      'operations' => array(),
    );
  }

  // Add/remove any actions from the stored script, comes back from AJAX calls.
  if (isset($form_state['triggering_element']['#action_id'])) {
    $triggering_element = $form_state['triggering_element'];

    // Get the operation and action ids from the triggering element.
    $operation_id = $form_state['triggering_element']['#operation_id'];
    $action_id = $form_state['triggering_element']['#action_id'];

    // Check if the triggering element has the remove_action defined,
    // then remove it, otherwise add it to the stored script.
    if (isset($triggering_element['#remove_action'])) {
      unset($form_state['storage']['script']['operations'][$operation_id][$action_id]);
    }
    else {
      $instance = restore_instance($operation_id);
      $form_state['storage']['script']['operations'][$operation_id][$action_id] = $instance->detailsFromId($action_id);
    }
  }

  // If the output has been generated, display the output instead.
  if (isset($form_state['storage']['output'])) {
    // The output text area.
    $form['output'] = array(
      '#type' => 'textarea',
      '#rows' => 15,
      '#value' => $form_state['storage']['output'],
      '#readonly' => TRUE,
    );

    // The output action buttons.
    $form['actions'] = array('#type' => 'actions');
    $form['actions']['back'] = array(
      '#type' => 'submit',
      '#submit' => array('restore_generate_form_back'),
      '#value' => t('Back'),
    );

    // Return the form.
    return $form;
  }

  // General details, such as title, and description.
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General'),
    '#collapsible' => TRUE,
  );

  // The scripts title.
  $form['general']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $form_state['storage']['script']['title'],
    '#required' => TRUE,
  );

  // The scripts description.
  $form['general']['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#default_value' => $form_state['storage']['script']['description'],
  );

  // The operations box container.
  $form['operations'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div id="restore-generate-container"><div id="restore-operations"><div class="inner">',
    '#suffix' => '</div></div>',
  );

  // The values/actions box container.
  $form['values'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div id="restore-values"><div class="inner">',
    '#suffix' => '</div></div><div style="clear:both;"></div></div>',
  );

  // The filters container.
  $form['values']['filter'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div id="restore-generate-filter">',
    '#suffix' => '</div>',
  );

  // Filter the id and title of the action.
  $form['values']['filter']['filter'] = array(
    '#type' => 'textfield',
    '#title' => t('Filter'),
    '#default_value' => isset($form_state['values']['values']['filter']['filter']) ? $form_state['values']['values']['filter']['filter'] : NULL,
    '#ajax' => array(
      'event' => 'change',
      'callback' => 'restore_generate_form_ajax_operation',
      'wrapper' => 'restore-values-items',
    ),
  );

  // Filter the selected actions.
  $form['values']['filter']['selected'] = array(
    '#type' => 'checkbox',
    '#title' => t('Selected'),
    '#default_value' => isset($form_state['values']['values']['filter']['selected']) ? $form_state['values']['values']['filter']['selected'] : FALSE,
    '#ajax' => array(
      'callback' => 'restore_generate_form_ajax_operation',
      'wrapper' => 'restore-values-items',
    ),
  );

  // Filter the unselected actions.
  $form['values']['filter']['unselected'] = array(
    '#type' => 'checkbox',
    '#title' => t('Unselected'),
    '#default_value' => isset($form_state['values']['values']['filter']['unselected']) ? $form_state['values']['values']['filter']['unselected'] : TRUE,
    '#ajax' => array(
      'callback' => 'restore_generate_form_ajax_operation',
      'wrapper' => 'restore-values-items',
    ),
  );

  // Container for all the operation action checkboxes.
  $form['values']['items'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div id="restore-values-items">',
    '#suffix' => '</div>',
  );

  // Get all the available operations.
  $operations = restore_operation();

  if (count($operations)) {
    // Cycle through all the available generate operations.
    foreach ($operations as $operation_id => $operation_info) {
      // Add the generate operation checkbox.
      $form['operations'][$operation_id] = array(
        '#type' => 'checkbox',
        '#title' => filter_xss($operation_info['label']),
        '#return_value' => $operation_id,
        '#ajax' => array(
          'callback' => 'restore_generate_form_ajax_operation',
          'wrapper' => 'restore-values-items',
        ),
      );

      // Check if the operation checkbox has been checked/ticked.
      if (!isset($form_state['values']['operations'][$operation_id]) || $form_state['values']['operations'][$operation_id] !== $operation_id) {
        continue;
      }

      // Get the operation instance.
      $operation = restore_instance($operation_id);

      // Get all the available options from the instance.
      $values = $operation->available();

      // Cycle through all the available options.
      $form['values']['items'][$operation_id] = array(
        '#tree' => TRUE,
        '#prefix' => '<div id="restore-actions-' . $operation_id . '" class="restore-actions-container"><h3>' . t('!label actions', array(
          '!label' => filter_xss($operation_info['label']),
        )) . '</h3>',
        '#suffix' => '</div>',
      );

      if (restore_generate_instance_use_ids($operation_id)) {
        $form['values']['items'][$operation_id]['#prefix'] .= '<h4 class="restore-has-ids">' . t('Warning: These actions use IDs which may be different on other systems.') . '</h4>';
      }

      foreach ($values as $id => $title) {
        // Check if this action is being filtered or not.
        if (isset($form_state['values']['values']['filter'])) {
          if (!empty($form_state['values']['values']['filter']['filter'])) {
            $filter = $form_state['values']['values']['filter']['filter'];
            if (strpos($id, $filter) === FALSE || strpos($title, $filter) === FALSE) {
              continue;
            }
          }

          if (
            (!empty($form_state['values']['values']['filter']['selected']) && empty($form_state['values']['values']['filter']['unselected']) && !isset($form_state['storage']['script']['operations'][$operation_id][$id])) ||
            (!empty($form_state['values']['values']['filter']['unselected']) && empty($form_state['values']['values']['filter']['selected']) && isset($form_state['storage']['script']['operations'][$operation_id][$id])) ||
            (empty($form_state['values']['values']['filter']['selected']) && empty($form_state['values']['values']['filter']['unselected']))
          ) {
            continue;
          }
        }

        // Add an action checkbox.
        $form['values']['items'][$operation_id][$id] = array(
          '#type' => 'checkbox',
          '#prefix' => '<div id="' . $operation_id . '--' . $id . '" class="restore-generate-action restore-generate-action-' . $operation_id . '">',
          '#suffix' => '</div>',
          '#title' => filter_xss($title, array(
            'a',
            'em',
            'strong',
            'cite',
            'blockquote',
            'code',
            'ul',
            'ol',
            'li',
            'dl',
            'dt',
            'dd',
            'br',
          )),
          '#return_value' => $id,
          '#default_value' => isset($form_state['storage']['script']['operations'][$operation_id][$id]) ? $id : NULL,
          '#operation_id' => $operation_id,
          '#action_id' => $id,
          '#ajax' => array(
            'callback' => 'restore_generate_form_ajax_action',
            'wrapper' => $operation_id . '--' . $id,
          ),
        );

        if (isset($form_state['storage']['script']['operations'][$operation_id][$id])) {
          $form['values']['items'][$operation_id][$id]['#remove_action'] = TRUE;
        }
      }
    }
  }

  // Restore script table view.
  $form['table'] = array(
    '#type' => 'fieldset',
    '#title' => 'Restore script',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['table']['table'] = array(
    '#type' => 'markup',
    '#prefix' => '<div id="restore-generate-table">',
    '#suffix' => '</div>',
    '#markup' => theme('restore_generate_table', array('script' => $form_state['storage']['script'])),
  );

  // "What to do with it" fieldset.
  $form['what_to_do_with_it'] = array(
    '#type' => 'fieldset',
    '#title' => t('What to do with it?'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  // The generate output button...
  $form['what_to_do_with_it']['export'] = array(
    '#type' => 'fieldset',
    '#title' => t("Exporting the script using the 'Export' button."),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['what_to_do_with_it']['export']['info'] = array(
    '#type' => 'markup',
    '#prefix' => '<p>',
    '#suffix' => '</p>',
    '#markup' => t('If you wish to just export the output of the script to manually edit an existing script, click on <b>Export</b>. This will provide the output code of the script which can be used in either a hook or an existing restore script file.'),
  );

  // The download script button...
  $form['what_to_do_with_it']['download'] = array(
    '#type' => 'fieldset',
    '#title' => t("Downloading the script using the 'Download' button."),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['what_to_do_with_it']['download']['info'] = array(
    '#type' => 'markup',
    '#prefix' => '<p>',
    '#suffix' => '</p>',
    '#markup' => t('If you wish to download the script file and place with other script files, click the <b>Download</b>, this will build the restore script and allow you to download it.'),
  );

  // The save to db button...
  $form['what_to_do_with_it']['database'] = array(
    '#type' => 'fieldset',
    '#title' => t("Saving the script using the 'Save to database' button."),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['what_to_do_with_it']['database']['info'] = array(
    '#type' => 'markup',
    '#prefix' => '<p>',
    '#suffix' => '</p>',
    '#markup' => t('The <b>Save to database</b> button will create/overwrite the restore script in the database, you can export the script to a file later using this generate UI.'),
  );

  // Form actions.
  $form['actions'] = array('#type' => 'actions');

  $form['actions']['generate'] = array(
    '#type' => 'submit',
    '#value' => t('Export'),
  );

  $form['actions']['generate_script_file'] = array(
    '#type' => 'submit',
    '#submit' => array('restore_generate_form_download'),
    '#value' => t('Download'),
  );

  $form['actions']['save_to_db'] = array(
    '#type' => 'submit',
    '#submit' => array('restore_generate_form_database'),
    '#value' => t('Save to database'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#submit' => array('restore_generate_form_cancel'),
    '#limit_validation_errors' => TRUE,
    '#value' => t('Cancel'),
  );

  return $form;
}

/**
 * Operation AJAX callback.
 *
 * Called when an operation checkbox is changed.
 */
function restore_generate_form_ajax_operation($form, &$form_state) {
  $commands = array();
  $commands[] = ajax_command_replace("#restore-values-items", render($form['values']['items']));

  return array(
    '#type' => 'ajax',
    '#commands' => $commands,
  );
}

/**
 * Action AJAX callback.
 *
 * Called when an action checkbox is changed.
 */
function restore_generate_form_ajax_action($form, &$form_state) {
  $triggering_element = $form_state['triggering_element'];
  $operation_id = $form_state['triggering_element']['#operation_id'];
  $action_id = $form_state['triggering_element']['#action_id'];

  $commands = array();
  $commands[] = ajax_command_replace("#{$operation_id}--{$action_id}", isset($form['values']['items'][$operation_id][$action_id]) ? render($form['values']['items'][$operation_id][$action_id]) : '');
  $commands[] = ajax_command_replace("#restore-generate-table", render($form['table']['table']));

  return array(
    '#type' => 'ajax',
    '#commands' => $commands,
  );
}

/**
 * The cancel button callback.
 *
 * Called when the 'Cancel' button is clicked.
 */
function restore_generate_form_cancel($form, &$form_state) {
  $form_state['redirect'] = 'admin/config/development/restore';
}

/**
 * The output back button callback.
 *
 * Called when the 'Back' button on the output screen is clicked.
 */
function restore_generate_form_back($form, &$form_state) {
  $form_state['rebuild'] = TRUE;
  unset($form_state['storage']['output']);
}

/**
 * Generate form submit callback.
 *
 * Called when the 'Generate' button is clicked.
 */
function restore_generate_form_submit($form, &$form_state) {
  $form_state['rebuild'] = TRUE;

  $form_state['storage']['script']['title'] = $form_state['values']['title'];
  $form_state['storage']['script']['description'] = $form_state['values']['description'];

  $script = restore_generate_to_script($form_state['storage']['script']);
  $form_state['storage']['output'] = restore_generate_script_to_text($script);
}

/**
 * Generate and download form submit callback.
 *
 * Called when the 'Download' button is clicked.
 */
function restore_generate_form_download($form, &$form_state) {
  $form_state['storage']['script']['title'] = $form_state['values']['title'];
  $form_state['storage']['script']['description'] = $form_state['values']['description'];

  $script = restore_generate_to_script($form_state['storage']['script']);

  $filename = drupal_strtolower(str_replace(' ', '_', $script['title'])) . '.script';

  header('Cache-Control: public');
  header('Content-Type: application/octet-stream');
  header('Cache-Control: no-store, no-cache, must-revalidate');
  header('Cache-Control: post-check=0, pre-check=0');
  header("Content-Disposition: attachment; filename=\"$filename\";");
  header('Content-Transfer-Encoding: binary');

  print restore_generate_script_to_text($script);

  drupal_exit();
}

/**
 * Save to database form submit callback.
 *
 * Called when the 'Save to database' button is clicked.
 */
function restore_generate_form_database($form, &$form_state) {
  $form_state['storage']['script']['title'] = $form_state['values']['title'];
  $form_state['storage']['script']['description'] = $form_state['values']['description'];

  $script = restore_generate_to_script($form_state['storage']['script']);
  if (restore_save_database_script($script)) {
    drupal_set_message(t('The script has been saved to the database successfully.'));
    $form_state['redirect'] = 'admin/config/development/restore/list/sid:' . $script['sid'];
  }
  else {
    $form_state['rebuild'] = TRUE;
    drupal_set_message(t('Failed to save the script to the database.'), 'error');
  }
}
