<?php
/**
 * @file
 * Operation plugin for module restore operations.
 */

$plugin = array(
  'label' => t('Module'),
  'handler' => array(
    'class' => 'RestoreModuleOperation',
  ),
);

class RestoreModuleOperation extends RestoreOperation {
  /** @var array Static storage for the modules during generate. */
  protected static $modules;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    // @TODO incorporate module versions?
    // $info = system_get_info('module', $action['name']);
    // dpm($info);
    return t("Module'<em>@module_name</em>'.", array('@module_name' => $action['name']));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    return module_exists($action['name']) ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    if (!module_exists($action['name'])) {
      module_enable($action['name']);
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['name'] !== $action['name']) {
        continue;
      }

      $results[$action_id] = t("Module '<em>@module_name</em>'.", array('@module_name' => $_action['name']));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $value = module_exists($action['name']);

    if (!$value) {
      return FALSE;
    }

    return array(
      'current_value' => $value ? t('Enabled') : t('Disabled'),
      'restore_value' => t('Enabled'),
      'match' => $value,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    return t("Module '<em>@module_name</em>'.", array('@module_name' => $id));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$modules)) {
      foreach (module_list() as $module_name) {
        static::$modules[$module_name] = $module_name;
      }
    }

    return static::$modules;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    return array(
      'name' => $id,
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    return array(
      'id' => $action['name'],
      'details' => $this->detailsFromId($action['name']),
    );
  }
}
