<?php
/**
 * @file
 * Operation plugin for roles restore operations.
 */

$plugin = array(
  'label' => t('Roles'),
  'handler' => array(
    'class' => 'RestoreRoleOperation',
  ),
);

class RestoreRoleOperation extends RestoreOperation {
  /** @var array Static storage for the roles for generate. */
  protected static $roles;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Role '<em>@role</em>'.", array(
      '@role' => $action['name'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $role = user_role_load_by_name($action['name']);
    return $role ? ($role->name == $action['name'] && $role->weight == $action['weight'] ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    $role = user_role_load_by_name($action['name']);
    if (!$role) {
      $role = (object) array();
    }

    $role->name = $action['name'];
    $role->weight = $action['weight'];
    user_role_save($role);

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    user_role_delete($action['name']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['name'] !== $action['name']) {
        continue;
      }

      $results[$action_id] = t("Role '<em>@role</em>'.", array('@role' => $_action['name']));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $role = user_role_load_by_name($action['name']);

    return array(
      'current_value' => $role ? $role->name : NULL,
      'restore_value' => $action['name'],
      'match' => $role && ($role->name == $action['name'] && $role->weight == $action['weight']),
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $role = user_role_load($id);
    if (!$role) {
      return FALSE;
    }

    return t("Role '<em>@role</em>'.", array('@role' => $role->name));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$roles)) {
      $roles = user_roles();

      foreach ($roles as $rid => $role) {
        static::$roles[$rid] = t("Role '<em>@role</em>'.", array('@role' => $role));
      }
    }

    return static::$roles;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $role = user_role_load($id);
    if (!$role) {
      return FALSE;
    }

    return array(
      'name' => $role->name,
      'weight' => $role->weight,
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $role = user_role_load_by_name($action['name']);
    if (!$role) {
      return FALSE;
    }

    return array(
      'id' => $action['name'],
      'details' => $this->detailsFromId($role->rid),
    );
  }
}
