<?php
/**
 * @file
 * Operation plugin for variable restore operations.
 */

$plugin = array(
  'label' => t('Variables'),
  'handler' => array(
    'class' => 'RestoreVariableOperation',
  ),
);

class RestoreVariableOperation extends RestoreOperation {
  /** @var array Static storage for the variables for generate. */
  protected static $variables;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Variable '<em>@variable_name</em>'.", array('@variable_name' => $action['name']));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    return isset($GLOBALS['conf'][$action['name']]) ? (variable_get($action['name']) == $action['value'] ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    variable_set($action['name'], $action['value']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    variable_del($action['name']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['name'] !== $action['name']) {
        continue;
      }

      $results[$action_id] = $this->details($_action);
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $exists = isset($GLOBALS['conf'][$action['name']]);
    $value = $exists ? variable_get($action['name']) : NULL;

    return array(
      'current_value' => $value,
      'restore_value' => $action['value'],
      'match' => $exists && $value == $action['value'],
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    return t("Variable '<em>@variable_name</em>'.", array('@variable_name' => $id));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$variables)) {
      $dbq = db_select('variable', 'v')
        ->fields('v', array('name'))
        ->execute();

      while ($row = $dbq->fetchAssoc()) {
        static::$variables[$row['name']] = $this->detailsFromId($row['name']);
      }
    }

    return static::$variables;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    return array(
      'name' => $id,
      'value' => variable_get($id),
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    return array(
      'id' => $action['name'],
      'details' => $this->detailsFromId($action['name']),
    );
  }
}
